
#include <stdio.h>
#include <glib.h> 

#include <gmyth.h>

static gboolean
_parse_args (int argc, char *argv[], GMythBackendInfo *b_info, char** filename)
{
    GError *error = NULL;
    GOptionContext *context;

    gchar *host_ip = NULL;
    gint host_port = 0;

    GOptionEntry entries[] = 
    {
        { "hostname", 'h', 0, G_OPTION_ARG_STRING, &host_ip, "Mythtv backend hostname or IP address", "IP_ADDRESS" },
        { "port", 'p', 0, G_OPTION_ARG_INT, &host_port, "Mythtv backend port", "PORT" },
        { "filename", 'f', 0, G_OPTION_ARG_STRING, filename, "Recorded file name available in the Mythtv backend", "FILE" },
        { NULL }
    };

    context = g_option_context_new ("- loads a mythtv backend recorded file and prints it on the standard output\n");
    g_option_context_add_main_entries (context, entries, NULL);
    g_option_context_parse (context, &argc, &argv, &error);
    g_option_context_set_help_enabled (context, TRUE);

    g_option_context_free (context);
    
    gmyth_backend_info_set_hostname (b_info, host_ip);
    gmyth_backend_info_set_port (b_info, host_port);

    g_free (host_ip);

    return TRUE;
}

int 
main (int argc, char *argv[])
{
    gboolean res = FALSE;
    gchar *filename = NULL;
    GMythBackendInfo *b_info;
    GByteArray *array = NULL;
    GMythFileTransfer *transfer;
    guint64 size = 0, total = 0;

    g_type_init ();
    g_thread_init (NULL);
    
    b_info = gmyth_backend_info_new ();

    res = _parse_args (argc, argv, b_info, &filename);
    if (!res) {
        g_printerr ("Argument invalid. Type --help\n");
        return 1;
    }

    if (!gmyth_util_file_exists (b_info, filename)) {
        g_printerr ("File %s was not found in the mythtv server\n", filename);
        return 1;
    }

    transfer = gmyth_file_transfer_new (b_info);
    if (!gmyth_file_transfer_open (transfer, filename)) {
        g_printerr ("File %s could not be opened\n", filename);
        return 1;
    }

    size = gmyth_file_transfer_get_filesize (transfer);
    while (total != size) {
        GMythFileReadResult res;

        array = g_byte_array_new ();
        res = gmyth_file_transfer_read (transfer, array, 
                                       (size - total) > 64000 ? 64000 : (size - total),
                                       FALSE);
        if ((res != GMYTH_FILE_READ_OK) && (res != GMYTH_FILE_READ_EOF)) {
           g_byte_array_free (array, TRUE);
           g_printerr ("Error while reading the file: aborting!!\n");
           break;
        }

        fwrite (array->data, array->len, 1, stdout);
        fflush (stdout);

        total += array->len;
        g_byte_array_free (array, TRUE);
    }
    
    g_free (filename);

    gmyth_file_transfer_close (transfer);
    g_object_unref (transfer);

    g_object_unref (b_info);

    return 0;
}
